/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * qrun
 *  The qrun command forces a batch job to run.
 *
 * Synopsis:
 *  qrun [-H host] job_identifier ...
 *
 * Arguments:
 *  host
 *      The host to run the job at.
 *  job_identifier ...
 *      A list of job_identifiers.  A job_identifier has the following form:
 *          sequence_number[.server_name][@server]
 *
 * Written by:
 *  Bruce Kelly
 *  National Energy Research Supercomputer Center
 *  Livermore, CA
 *  May, 1993
 */

#include "cmds.h"
#include <pbs_config.h>   /* the master config generated by configure */

static char ident[] = "@(#) $RCSfile: qrun.c,v $ $Revision: 2.0 $";

int exitstatus = 0; /* Exit Status */
static void execute ();



main ( argc, argv )
int argc;
char **argv;
{
/*
 *  This routine sends a Run Job request to the batch server.  If the
 * batch request is accepted, the server will have started the execution
 * of the job.
 */

    int id;                         /* Index into the job_id array (argv) */
    char job[PBS_MAXCLTJOBID];      /* Job Id */
    char server[MAXSERVERNAME];	    /* Server name */
    char *location = NULL;          /* Where to run the job */
    
    static char opts[] = "H:";     /* See man getopt */
    static char *usage = "Usage: qrun [-H host] job_id ...\n";
    int s;
    int errflg = 0;
    
    /* Command line options */
    while ((s = getopt(argc, argv, opts)) != EOF)
        switch (s) {
        case 'H':
	    if ( strlen(optarg) == 0 ) {
		fprintf(stderr, "qrun: illegal -H value\n");
		errflg++;
		break;
	    }
            location = optarg;
            break;
        case '?':
        default:
            errflg++;
            break;
        }

    if ( errflg || (optind >= argc) ) {
        fprintf(stderr, usage);
        exit(1);
    }
    
    for ( ; optind < argc; optind++ ) {
        if ( get_server(argv[optind], job, server) ) {
            fprintf(stderr, "qrun: illegally formed job identifier: %s\n", argv[optind]);
	    exitstatus = 1;
	    continue;
	}
        execute(job, server, location);
    }
    exit (exitstatus);
}


/*
 * void execute( char *job, char *server, char *location )
 *
 * job      The fully qualified job id.
 * server   The name of the server that manages the job.
 * location The name of the server to run the job.
 *
 * Returns:
 *  None
 *
 * File Variables:
 *  exitstatus  Set to two if an error occurs.
 */
static void
execute( job, server, location )
char *job;
char *server;
char *location;
{
    int ct;         /* Connection to the server */
    int err;        /* Error return from pbs_run */
    char *errmsg;   /* Error message from pbs_run */
    int located = FALSE;
    char rmt_server[MAXSERVERNAME];

cnt:    
    if ( (ct = cnt2server(server)) > 0 ) {
        err = pbs_runjob(ct, job, location, NULL);
        if ( err && (pbs_errno != PBSE_UNKJOBID) ) {
	    prt_job_err("qrun", ct, job);
            exitstatus = 2;
        } else if ( err && (pbs_errno == PBSE_UNKJOBID) && !located ) {
	    located = TRUE;
	    if ( locate_job(job, server, rmt_server) ) {
	        pbs_disconnect(ct);
		strcpy(server, rmt_server);
		goto cnt;
	    }
	    prt_job_err("qrun", ct, job);
            exitstatus = 2;
	}
        pbs_disconnect(ct);
    } else {
        fprintf(stderr, "qrun: could not connect to server %s (%d)\n", server, pbs_errno);
        exitstatus = 2;
    }
}
